##############################################################################
#
# Copyright (c) 2004 Dinheiro Vivo <www.dinheirovivo.com.br>
#                    by Jean Rodrigo Ferri <jeanrodrigoferri@yahoo.com.br>
# 
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
# 
##############################################################################

import os, string
from Globals import InitializeClass, DTMLFile, package_home
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName, UniqueObject
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression
from PublicationBoxInformation import PublicationBoxInformation

from Products.CMFCore.CMFCorePermissions import ManagePortal, ReviewPortalContent, View

_wwwdir = os.path.join( package_home( globals() ), 'www' )

# Uncomment/comment the bellow lines if you want copy or create
# more portal_publicator tool in other nodes of site.
#class PublicatorTool(SimpleItem, ActionProviderBase):
class PublicatorTool(UniqueObject, SimpleItem, ActionProviderBase):
    """ A tool that provide an mechanism that store metadata items and it allows
        the item publication only that the reviewer like.

        Oposed to default CMF/Plone publication machinary.
    """

    meta_type='CMF Publicator'
    id = 'portal_publicator'
    title = 'Content Publicator'

    _publication_boxes = ()

    ##############
    # Action items

    _actions = (ActionInformation(id='edit_publications',
                                  title='Publications',
                                  action=Expression(text='string:${portal_url}/portal_publicator/publicator_setup_form'),
                                  condition=Expression(text='member'),
                                  permissions=(ReviewPortalContent,),
                                  category='user',
                                  visible=1
                                 ),
                ActionInformation(id='edit_publications',
                                  title='Publications',
                                  action=Expression(text='python:object.portal_publicator.absolute_url()+"/publicator_setup_form"'),
                                  condition=Expression(text='python:member and folder.portal_publicator.absolute_url()==folder.absolute_url()+"/portal_publicator"'),
                                  permissions=(ReviewPortalContent,),
                                  category='folder',
                                  visible=0
                                 )
               )
    ##########
    # ZMI tabs

    _publication_boxes_form = DTMLFile('editPublicationBoxes', _wwwdir)
    _publication_boxes_items_form = DTMLFile('viewPublicationBoxesItems', _wwwdir)

    manage_overview = PageTemplateFile('portal_publicator_overview', _wwwdir)

    manage_options = ( {'label':'Overview', 'action':'manage_overview'},
                       {'label':'Configuration', 'action':'manage_editPublicationBoxesForm'},
                       {'label':'Stored items', 'action':'manage_viewPublicationBoxesItemsForm'}
                     ) + ActionProviderBase.manage_options + SimpleItem.manage_options

    security = ClassSecurityInfo()

    security.declarePrivate('listPublicationBoxes')
    def listPublicationBoxes(self, pb=None):

        """ Return all the publication boxes or a specified pb.
        """
        if pb is None:
            return self._publication_boxes or ()
        else:
            return self._publication_boxes[pb] or ()

    #############
    # ZMI methods

    security.declareProtected( ManagePortal, 'manage_editPublicationBoxesForm' )
    def manage_editPublicationBoxesForm( self, REQUEST, manage_tabs_message=None ):

        """ Show the 'Configuration' management tab.
        """
        publication_boxes = []

        for pb in self.listPublicationBoxes():
            pb_dict = {}
            pb_dict['id'] = pb.getId()
            pb_dict['name'] = pb.getName()
            pb_dict['content_type'] = pb.getContent_type()
            pb_dict['icon_url'] = pb.getIcon_url()
            pb_dict['n_items'] = pb.getN_items()
            pb_dict['n_searched_items'] = pb.getN_searched_items()
            pb_dict['search_states'] = pb.getSearch_states()
            pb_dict['with_image'] = pb.getWith_image()
            pb_dict['images_folder'] = pb.getImages_folder()
            pb_dict['image_states'] = pb.getImage_states()
            pb_dict['box_width'] = pb.getBox_width()
            pb_dict['box_height'] = pb.getBox_height()
            pb_dict['visible'] = pb.getVisible()
            pb_dict['items'] = pb.getItems()
            publication_boxes.append(pb_dict)

        return self._publication_boxes_form( self,
                                             REQUEST,
                                             publication_boxes=publication_boxes,
                                             management_view='Configuration',
                                             manage_tabs_message=manage_tabs_message
                                           )

    security.declareProtected( ManagePortal, 'manage_viewPublicationBoxesItemsForm' )
    def manage_viewPublicationBoxesItemsForm( self, REQUEST, manage_tabs_message=None ):

        """ Show the 'Stored items' tab.
        """
        publication_boxes_items = []

        for pb in self.listPublicationBoxes():
            pb_dict = {}
            pb_dict['id'] = pb.getId()
            pb_dict['items'] = pb.getItems()
            publication_boxes_items.append(pb_dict)

        return self._publication_boxes_items_form( self,
                                                   REQUEST,
                                                   publication_boxes_items=publication_boxes_items,
                                                   management_view='Stored items',
                                                   manage_tabs_message=manage_tabs_message
                                                 )

    security.declareProtected( ManagePortal, 'addPublicationBox' )
    def addPublicationBox( self,
                           id,
                           name,
                           content_type,
                           icon_url='',
                           n_items=1,
                           n_searched_items=30,
                           search_states=['published'],
                           with_image=0,
                           images_folder='',
                           image_states=['visible'],
                           box_width='auto',
                           box_height='auto',
                           visible=1,
                           REQUEST=None
                         ):

        """ Add an publication box to the list.
        """
        if not id:
            raise ValueError('A id is required.')

        if not name:
            raise ValueError('A name is required.')

        if not content_type:
            raise ValueError('A content type is required.')

        new_publicationBoxes = self._clonePublicationBoxes()

        new_publicationBox = PublicationBoxInformation( id=str(id),
                                                        title=str(name),
                                                        content_type=list(content_type),
                                                        icon_url=str(icon_url),
                                                        n_items=int(n_items),
                                                        n_searched_items=int(n_searched_items),
                                                        search_states=list(search_states),
                                                        with_image=int(with_image),
                                                        images_folder=str(images_folder),
                                                        image_states=list(image_states),
                                                        box_width=str(box_width),
                                                        box_height=str(box_height),
                                                        visible=int(visible)
                                                      )

        new_publicationBoxes.append( new_publicationBox )
        self._publication_boxes = tuple( new_publicationBoxes )

        if REQUEST is not None:
            return self.manage_editPublicationBoxesForm(
                REQUEST, manage_tabs_message='Publication box added.')

    security.declareProtected( ManagePortal, 'changePublicationBoxes' )
    def changePublicationBoxes( self, properties=None, REQUEST=None ):

        """ Update our list of publication boxes.
        """
        if properties is None:
            properties = REQUEST

        publication_boxes = []

        for index in range( len( self._publication_boxes ) ):
            publication_boxes.append( self._extractPublicationBox( properties, index ) )

        self._publication_boxes = tuple( publication_boxes )

        if REQUEST is not None:
            return self.manage_editPublicationBoxesForm(REQUEST, manage_tabs_message=
                                                           'Publication boxes changed.')

    security.declareProtected( ReviewPortalContent, 'editPBItems' )
    def editPBItems( self, properties=None, REQUEST=None ):

        """ Update our list of items of all Publication Boxes.
        """
        status_msg = 'Publication boxes updated.'

        if properties is None:
            properties = REQUEST

        for box in range( self.getSize() ):
            self.changePBItems( box, properties )

        if REQUEST is not None:
            self.REQUEST.RESPONSE.redirect(self.index_html.absolute_url()+'?portal_status_message='+string.replace(status_msg,' ','+'))

    security.declareProtected( ReviewPortalContent, 'changePBItems' )
    def changePBItems( self, box, properties ):

        """ Update our list of items of an specific Publication Box.
        """
        pb = self.getPublicationBox(box)
        items = []

        for item in range( pb.getN_items() ):
            items.append( self._extractPBItems( properties, box, item ) )

        pb.items = items[:]

    security.declareProtected( ReviewPortalContent, 'resetPBItems' )
    def resetPBItems( self, REQUEST=None ):

        """ Reset our list of items from each Publication Box.
        """
        boxes = self.listPublicationBoxes()

        for pb in boxes:
            pb.items = []

        if REQUEST is not None:
            return self.manage_viewPublicationBoxesItemsForm(REQUEST, manage_tabs_message=
                                                                      'Publication boxes reseted.')

    security.declareProtected( ManagePortal, 'deletePublicationBoxes' )
    def deletePublicationBoxes( self, selections=(), REQUEST=None ):

        """ Delete publication boxes indicated by indexes in 'selections'.
        """
        sels = list( map( int, selections ) )  # Convert to a list of integers.

        old_publication_boxes = self._clonePublicationBoxes()
        new_publication_boxes = []

        for index in range( len( old_publication_boxes ) ):
            if index not in sels:
                new_publication_boxes.append( old_publication_boxes[ index ] )

        self._publication_boxes = tuple( new_publication_boxes )

        if REQUEST is not None:
            return self.manage_editPublicationBoxesForm(REQUEST, manage_tabs_message=(
                   'Deleted %d publication box(es).' % len(sels)))

    security.declarePrivate( '_clonePublicationBoxes' )
    def _clonePublicationBoxes( self ):

        """ Return a list of publication boxes, cloned from our current list.
        """
        return map( lambda x: x.clone(), list( self._publication_boxes ) )
 
    security.declarePrivate( '_extractPublicationBox' )
    def _extractPublicationBox( self, properties, index ):

        """ Extract an PublicationBoxInformation from the funky form properties.
        """
        id               = str( properties.get( 'id_%d'               % index, '' ) )
        name             = str( properties.get( 'name_%d'             % index, '' ) )
        content_type     = list(properties.get( 'content_type_%d'     % index, '' ) )
        icon_url         = str( properties.get( 'icon_url_%d'         % index, '' ) )
        n_items          = int( properties.get( 'n_items_%d'          % index, 1  ) )
        n_searched_items = int( properties.get( 'n_searched_items_%d' % index, 30 ) )
        search_states    = list(properties.get( 'search_states_%d'    % index, '' ) )
        with_image       = int( properties.get( 'with_image_%d'       % index, 0  ) )
        images_folder    = str( properties.get( 'images_folder_%d'    % index, '' ) )
        image_states     = list(properties.get( 'image_states_%d'     % index, '' ) )
        box_width        = str( properties.get( 'box_width_%d'        % index, '' ) )
        box_height       = str( properties.get( 'box_height_%d'       % index, '' ) )
        visible          = int( properties.get( 'visible_%d'          % index, 1  ) )

        if not id:
            raise ValueError('A id is required.')

        if not name:
            raise ValueError('A name is required.')

        if not content_type:
            raise ValueError('A content type is required.')

        pb = self.getPublicationBox(index)
        items = pb.items[:]

        return PublicationBoxInformation( id=id,
                                          title=name,
                                          content_type=content_type,
                                          icon_url=icon_url,
                                          n_items=n_items,
                                          n_searched_items=n_searched_items,
                                          search_states=search_states,
                                          with_image=with_image,
                                          images_folder=images_folder,
                                          image_states=image_states,
                                          box_width=box_width,
                                          box_height=box_height,
                                          visible=visible,
                                          items=items
                                        )

    security.declarePrivate( '_extractPBItems' )
    def _extractPBItems( self, properties, box, item ):

        """ Return the Item attributes to the received Item properties form extracted.
        """
        item_metadata = {}

        RelativeURL = str( properties.get( 'item_%d_%d' % (box, item), '' ) )
        item_metadata['RelativeURL'] = RelativeURL
        item_metadata['ImageURL'] = str( properties.get( 'image_url_%d_%d' % (box, item), '' ) )
        item_metadata['NewWindow'] = str( properties.get( 'new_window_%d_%d' % (box, item), '' ) )

        if not RelativeURL:
            object = None
        else:
            try:
                portal_object = getToolByName(self, 'portal_url').getPortalObject()
                object = portal_object.restrictedTraverse(RelativeURL)
            except:
                raise ValueError('A inconsistent path has been passed in item restrictedTraverse.')

        # Default DublinCore properties
        try:
            item_metadata['Title'] = object.Title()
            item_metadata['Creator'] = object.Creator()
            item_metadata['Subject'] = object.Subject()
            item_metadata['Description'] = object.Description()
            item_metadata['Contributors'] = object.Contributors()
            item_metadata['Date'] = object.Date()
            item_metadata['CreationDate'] = object.CreationDate()
            item_metadata['EffectiveDate'] = object.EffectiveDate()
            item_metadata['ExpirationDate'] = object.ExpirationDate()
            item_metadata['ModificationDate'] = object.ModificationDate()
            item_metadata['Type'] = object.Type()
            item_metadata['Format'] = object.Format()
            item_metadata['Identifier'] = object.Identifier()
            item_metadata['Language'] = object.Language()
            item_metadata['Rights'] = object.Rights()
        except:
            item_metadata['Title'] = ''
            item_metadata['Creator'] = ''
            item_metadata['Subject'] = None
            item_metadata['Description'] = ''
            item_metadata['Contributors'] = None
            item_metadata['Date'] = None
            item_metadata['CreationDate'] = None
            item_metadata['EffectiveDate'] = None
            item_metadata['ExpirationDate'] = None
            item_metadata['ModificationDate'] = None
            item_metadata['Type'] = ''
            item_metadata['Format'] = ''
            item_metadata['Identifier'] = ''
            item_metadata['Language'] = ''
            item_metadata['Rights'] = ''

        # Link property
        try:
            item_metadata['RemoteUrl'] = object.getRemoteUrl()
        except:
            item_metadata['RemoteUrl'] = ''

        # Event property
        try:
            item_metadata['start'] = object.start()
            item_metadata['end'] = object.end()
        except:
            item_metadata['start'] = None
            item_metadata['end'] = None

        # Zope properties
        try:
            item_metadata['Id'] = object.getId()
        except:
            item_metadata['Id'] = ''
        try:
            item_metadata['Icon'] = object.getIcon(relative_to_portal=1)
        except:
            item_metadata['Icon'] = ''

        # Customize the getItemInfo Python Script skin to return other
        # needed metadatas. Ex.: Specific metadata Archetypes content.
        try:
            if object is not None:
                external_info = self.getItemInfo(object)
            else:
                # It need return a dictionary with empty keys.
                external_info = self.getItemInfo()
            for item in external_info.items():
                item_metadata[item[0]] = item[1]
        except:
            raise ValueError('Error in getItemInfo method. It need return a dictionary.')

        return item_metadata

    ####################
    # Iteractive methods

    security.declareProtected( View, 'getPublicationBoxes' )
    def getPublicationBoxes(self):

        """ Return all the publication boxes content.
        """
        publication_boxes = []

        for pb in self.listPublicationBoxes():
            pb_dict = {}
            pb_dict['id'] = pb.getId()
            pb_dict['name'] = pb.getName()
            pb_dict['content_type'] = pb.getContent_type()
            pb_dict['icon_url'] = pb.getIcon_url()
            pb_dict['n_items'] = pb.getN_items()
            pb_dict['n_searched_items'] = pb.getN_searched_items()
            pb_dict['search_states'] = pb.getSearch_states()
            pb_dict['with_image'] = pb.getWith_image()
            pb_dict['images_folder'] = pb.getImages_folder()
            pb_dict['image_states'] = pb.getImage_states()
            pb_dict['box_width'] = pb.getBox_width()
            pb_dict['box_height'] = pb.getBox_height()
            pb_dict['visible'] = pb.getVisible()
            pb_dict['items'] = pb.getItems()

            publication_boxes.append(pb_dict)

        return publication_boxes

    security.declareProtected( View, 'getPublicationBox' )
    def getPublicationBox(self, box):

        """ Return the specified publication box.
        """
        return self.listPublicationBoxes(box)

    security.declareProtected( View, 'getPublicationBoxById' )
    def getPublicationBoxById(self, id, field=None):

        """ Return publication box if have the specified id.
        """
        if field is None:
            for pb in self.getPublicationBoxes():
                if pb['id'] == id:
                    return pb
        else:
            try:
                for pb in self.getPublicationBoxes():
                    if pb['id'] == id:
                        return pb[field]
            except:
                raise ValueError('The field is not a valid parameter.')

    security.declareProtected( View, 'getSize' )
    def getSize(self):

        """ Return the size of publication boxes tuple.
        """
        return len(self._publication_boxes)

    security.declareProtected( View, 'getItemsContent' )
    def getItemsContent(self, items, index):

        """ Return the item content the from items list or empty dict.
        """
        items_size = len(items)
        if index < items_size:
            # Item has data
            return items[index]
        else:
            item_content = {}
            item_content['RelativeURL'] = ''
            item_content['NewWindow'] = None
            item_content['ImageURL'] = ''
            return item_content
            
    security.declareProtected( View, 'getImagesSearched' )
    def getImagesSearched(self, need_image, relative_portal_path, content_type, content_state, relative=None):

        """ Return the searched catalog images path and id, if the item need it.
        """
        if need_image is not None:
            result = []
            catalog = getToolByName(self, 'portal_catalog')
            portal_url = getToolByName(self, 'portal_url')
            portal_path = portal_url.getPortalPath()
            relative_path = portal_path+'/'+relative_portal_path

            query = catalog.searchResults(path=relative_path,
                                          portal_type=content_type,
                                          sort_on='Date',
                                          sort_order='reverse',
                                          review_state=content_state)

            if relative is not None:
                portal_path_length = len(portal_url.getPortalObject().getPhysicalPath())
                # Extract the relative URL from cataloged items
                for obj in query:
                    content_dict = {}
                    content_path = obj.getPath().split('/')
                    relative_content_path = content_path[portal_path_length:]
                    content_dict['relative_url'] = '/'.join(relative_content_path)
                    content_dict['id'] = obj.id
                    result.append(content_dict)
            else:
                result = query

        else:
            result = None

        return result

    security.declareProtected( View, 'getListTypeInfo' )
    def getListTypeInfo(self):

        """ Return the list type info in CMF portal_types.
        """
        result = []
        for item in getToolByName(self, 'portal_types').listTypeInfo():
            result.append(str(item.getId()))

        return result

InitializeClass( PublicatorTool )

